/*********************************************************************
* UNRLE.C  -  This program will decompresses a file was RLE encoded  *
* by the sister program RLE.C. It reacts to the 0x90 RLE repeat      *
* identifier byte by expanding encoded runs back into their original *
* form.                                                              *
*                                                                    *
* To compile: CL UNRLE.C                                             *
* To run : UNRLE <infile> <outfile>                                  *
*                                                                    *
* Author  : Gary Conway                                              *
* Last Revised : 7-15-91 by David Reid                               *
*********************************************************************/

#include <stdio.h>             /* printf(), fopen(), fgetc(), ...   */
#include <stdlib.h>            /* exit(), EXIT_FAILURE              */

void Decompress( void );       /* Prototype for decompressor        */

/*----Program constants---------------------------------------------*/
#define Repeat      0x90       /* RLE repeat sequence identifier    */
#define FirstTime   0          /* no relevant history               */
#define Repeating   1          /* sending a repeated value          */

/*----Global variables----------------------------------------------*/
int c;                         /* Input character                   */
int state;                     /* Current machine state             */
FILE *infile, *outfile;        /* input and output files            */

/*********************************************************************
* main - Main body of program                                        *
*********************************************************************/
void main( int argc, char **argv )
    {
    /*----Check for two command line arguments----------------------*/
    if ( argc != 3 )
        {
        puts( "SYNTAX: rle <infile> <outfile>" );
        exit( EXIT_FAILURE );
        }

    /*----Open the input file---------------------------------------*/
    if ( (infile = fopen( argv[1], "rb" )) == NULL )
        {
        printf( "Unable to open input file: %s\n", argv[1] );
        exit( EXIT_FAILURE );
        }

    /*----See if output file already exists-------------------------*/
    if ( (outfile = fopen( argv[2], "rb" )) != NULL )
        {
        printf( "Output file %s already exists", argv[2] );
        fclose( infile );
        fclose( outfile );
        exit( EXIT_FAILURE );
        }

    /*----Open the output file--------------------------------------*/
    if ( (outfile = fopen( argv[2], "wb" )) == NULL )
        {
        printf( "Unable to open output file: %s\n", argv[2] );
        fclose( infile );
        exit( EXIT_FAILURE );
        }

    while ( ( c = fgetc(infile) ) != EOF )
        Decompress();

    fclose( infile );
    fclose( outfile );
    }

/*********************************************************************
* The RLE Decoding state machine                                     *
*********************************************************************/
void Decompress( void )
    {
    static int lastc;              /* The last character read       */

    switch(state)
        {
        case FirstTime:
            if ( c == Repeat )     /* If starting a new series      */
                state = Repeating; /* then remember it next time    */
            else                   /* else output as normal         */
                fputc( lastc = c, outfile);
            return;

        case Repeating:
            if ( c )               /* If count is nonzero           */
                {
                c += 2;
                while ( c-- )      /* then expand the sequence      */
                    fputc( lastc, outfile );
                }
            else                   /* else output RLE as data byte  */
                fputc( Repeat, outfile );
            state = FirstTime;     /* Start over with new run/char  */
            return;
        }
    }

